@@ -1,25 +1,23 @@
 """
     DigiCloud Access Service.
 """
-from cliff.lister import Lister
-from cliff.show import ShowOne
-from cliff.command import Command
 
-from ..utils import tabulate
+from .base import Lister, ShowOne, Command
+from .. import schemas
 
 
 class ListFloatingIp(Lister):
     """List floating Ips."""
+    schema = schemas.FloatingIPList(many=True)
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/floating-ips'
-        data = self.app.session.get(uri)
-
-        return tabulate(data)
+        return self.app.session.get(uri)
 
 
 class ShowFloatingIp(ShowOne):
     """Show floating Ip details."""
+    schema = schemas.FloatingIPDetails()
 
     def get_parser(self, prog_name):
         parser = super(ShowFloatingIp, self).get_parser(prog_name)
@@ -30,19 +28,17 @@ class ShowFloatingIp(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/floating-ips/%s' % parsed_args.floating_ip
-        data = self.app.session.get(uri)
-
-        return tabulate(data)
+        return self.app.session.get(uri)
 
 
 class CreateFloatingIp(ShowOne):
     """Create Floating Ip."""
-    def take_action(self, parsed_args):
-        data = self.app.session.post('/floating-ips', {})
+    schema = schemas.FloatingIPDetails()
 
-        return tabulate(data)
+    def get_data(self, parsed_args):
+        return self.app.session.post('/floating-ips', {})
 
 
 class DeleteFloatingIp(Command):
@@ -64,6 +60,7 @@ class DeleteFloatingIp(Command):
 
 class AssociateFloatingIp(ShowOne):
     """Associate Floating Ip."""
+    schema = schemas.FloatingIPDetails()
 
     def get_parser(self, prog_name):
         parser = super(AssociateFloatingIp, self).get_parser(prog_name)
@@ -80,17 +77,17 @@ class AssociateFloatingIp(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         payload = {
             'interface_id': parsed_args.interface_id,
         }
         uri = '/floating-ips/%s/associate' % parsed_args.floating_ip_id
-        data = self.app.session.post(uri, payload)
-        return tabulate(data)
+        return self.app.session.post(uri, payload)
 
 
 class RevokeFloatingIp(ShowOne):
     """Revoke Floating Ip."""
+    schema = schemas.FloatingIPDetails()
 
     def get_parser(self, prog_name):
         parser = super(RevokeFloatingIp, self).get_parser(prog_name)
@@ -102,7 +99,6 @@ class RevokeFloatingIp(ShowOne):
 
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/floating-ips/%s/revoke' % parsed_args.floating_ip_id
-        data = self.app.session.post(uri, {})
-        return tabulate(data)
+        return self.app.session.post(uri, {})
