@@ -1,17 +1,18 @@
 """
     DigiCloud Compute Instance Service.
 """
+from rich.prompt import Confirm
 
-from cliff.command import Command
-from cliff.lister import Lister
-from cliff.show import ShowOne
-
-from .. import formatter
-from ..utils import tabulate, is_tty
+from digicloud import schemas
+from .base import Lister, ShowOne, Command
+from ..error_handlers import CLIError
+from ..utils import is_tty
 
 
 class ListInstance(Lister):
     """List instances."""
+    help_file = 'instance.txt'
+    schema = schemas.InstanceList(many=True)
 
     def get_parser(self, prog_name):
         parser = super(ListInstance, self).get_parser(prog_name)
@@ -31,10 +32,11 @@ class ListInstance(Lister):
 
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         args = (parsed_args.simple, parsed_args.advanced)
         if all(args):
-            raise Exception("You need to specify one of --simple or --advanced")
+            raise CLIError(
+                [dict(msg="You need to specify one of --simple or --advanced")])
 
         query_params = {}
         if parsed_args.simple:
@@ -43,15 +45,13 @@ class ListInstance(Lister):
             query_params['type'] = 'advanced'
 
         data = self.app.session.get( '/instances', params=query_params)
-
-        for instance in data:
-            instance['memory'] = formatter.format_ram(int(instance['memory']))
-
-        return tabulate(data)
+        return data
 
 
 class CreateInstance(ShowOne):
     """Create instance."""
+    help_file = 'instance.txt'
+    schema = schemas.InstanceDetails()
 
     def get_parser(self, prog_name):
         parser = super(CreateInstance, self).get_parser(prog_name)
@@ -90,11 +90,11 @@ class CreateInstance(ShowOne):
         )
 
         parser.add_argument(
-            '--security-group',
-            metavar='<security_group>',
+            '--firewall',
+            metavar='<firewall>',
             default=[],
             action='append',
-            help='Security group name or ID'
+            help='Firewall name or ID'
         )
         parser.add_argument(
             '--simple',
@@ -129,15 +129,15 @@ class CreateInstance(ShowOne):
         rules = [
             (
                 all((parsed_args.advanced, parsed_args.simple)),
-                "--advanced and --simple should not be used together",
+                "--advanced and --simple must not be used together",
             ),
             (
                 parsed_args.simple and parsed_args.network is not None,
-                "--simple and --network should not be used together",
+                "--simple and --network must not be used together",
             ),
             (
-                parsed_args.simple and parsed_args.security_group,
-                "--simple and --security-group should not be used together",
+                parsed_args.simple and parsed_args.firewall,
+                "--simple and --firewall must not be used together",
             ),
             (
                 parsed_args.advanced and parsed_args.network is None,
@@ -149,12 +149,14 @@ class CreateInstance(ShowOne):
             ),
 
         ]
-
+        errors = []
         for is_invalid, err_msg in rules:
             if is_invalid:
-                raise Exception(err_msg)
+                errors.append(dict(msg=err_msg))
+        if errors:
+            raise CLIError(errors)
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         self._check_arg_validity(parsed_args)
         payload = {
             'name': parsed_args.name,
@@ -166,8 +168,8 @@ class CreateInstance(ShowOne):
             payload['network'] = parsed_args.network
         if parsed_args.ssh_key:
             payload['ssh_key_name'] = parsed_args.ssh_key
-        if parsed_args.security_group:
-            payload['security_groups'] = parsed_args.security_group
+        if parsed_args.firewall:
+            payload['security_groups'] = parsed_args.firewall
         if parsed_args.additional_volumes:
             payload['additional_volumes'] = [
                 {
@@ -181,14 +183,13 @@ class CreateInstance(ShowOne):
             payload['description'] = parsed_args.description
 
         data = self.app.session.post('/instances', payload)
-
-        data['memory'] = formatter.format_ram(int(data['memory']))
-
-        return tabulate(data)
+        return data
 
 
 class ShowInstance(ShowOne):
     """Show instance details."""
+    help_file = 'instance.txt'
+    schema = schemas.InstanceDetails()
 
     def get_parser(self, prog_name):
         parser = super(ShowInstance, self).get_parser(prog_name)
@@ -199,17 +200,15 @@ class ShowInstance(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/instances/%s' % parsed_args.instance
         data = self.app.session.get(uri)
-
-        data['memory'] = formatter.format_ram(int(data['memory']))
-
-        return tabulate(data)
+        return data
 
 
 class DeleteInstance(Command):
     """Delete a instance."""
+    help_file = 'instance.txt'
 
     def get_parser(self, prog_name):
         parser = super(DeleteInstance, self).get_parser(prog_name)
@@ -224,6 +223,13 @@ class DeleteInstance(Command):
             help='Instance name or ID',
         )
 
+        parser.add_argument(
+            '--delete-root-volume',
+            help='Use this switch to also delete root volume',
+            default=None,
+            action='store_true'
+        )
+
         parser.add_argument(
             '--i-am-sure',
             help='Use this switch to bypass confirmation',
@@ -234,134 +240,47 @@ class DeleteInstance(Command):
         return parser
 
     def take_action(self, parsed_args):
-        if not self.confirm(parsed_args):
+        if not self.confirm_instance_deletion(parsed_args):
             return
-        if parsed_args.delete_volumes:
-            volumes_uri = '/instances/%s/volumes' % parsed_args.instance
-            volumes = self.app.session.get(volumes_uri)
-            for volume in volumes:
-                del_vol_uri = '/instances/%s/volumes/%s' % parsed_args.instance, volume['id']
-                self.app.session.delete(del_vol_uri)
-
         uri = '/instances/%s' % parsed_args.instance
+        if self.confirm_volume_deletion(parsed_args):
+            uri += "?delete_root_volume=true"
         self.app.session.delete(uri)
 
-    def confirm(self, parsed_args):
+    def confirm_instance_deletion(self, parsed_args):
         if parsed_args.i_am_sure:
             return True
         if is_tty():
             instance = self.app.session.get('/instances/%s' % parsed_args.instance)
-            user_response = input(
-                "You're about to delete the instance named {}  "
-                "Are you sure? [yes/no]".format(
+            user_response = Confirm.ask(
+                "You're about to delete the instance named [red bold]{}[/red bold]. "
+                "Are you sure?".format(
                     instance['name']
-                ))
-            if user_response == "yes":
+                ), default=False
+            )
+            if user_response:
                 return True
             self.app.stdout.write("Operation cancelled by user\n")
         else:
-            self.app.stderr.write("Unable to perform 'instance delete' operation in non-interactive mode,"
-                                  " without '--i-am-sure' switch\n")
+            self.app.stderr.write(
+                "Unable to perform 'instance delete' operation in non-interactive mode,"
+                " without '--i-am-sure' switch\n")
             return False
 
-
-class StartInstance(Command):
-    """Start instance."""
-
-    def get_parser(self, prog_name):
-        parser = super(StartInstance, self).get_parser(prog_name)
-        parser.add_argument(
-            'instance',
-            metavar='<instance>',
-            help='Instance name or ID',
-        )
-        return parser
-
-    def take_action(self, parsed_args):
-        uri = '/instances/%s/state-transitions' % parsed_args.instance
-        self.app.session.post(uri, payload=dict(new_state="ACTIVE"))
-
-
-class StopInstance(Command):
-    """Stop instance."""
-
-    def get_parser(self, prog_name):
-        parser = super(StopInstance, self).get_parser(prog_name)
-        parser.add_argument(
-            'instance',
-            metavar='<instance>',
-            help='Instance name or ID',
-        )
-        return parser
-
-    def take_action(self, parsed_args):
-        uri = '/instances/%s/state-transitions' % parsed_args.instance
-        self.app.session.post(uri, payload=dict(new_state="SHUTOFF"))
-
-
-class SuspendInstance(Command):
-    """Suspend instance."""
-
-    def get_parser(self, prog_name):
-        parser = super(SuspendInstance, self).get_parser(prog_name)
-        parser.add_argument(
-            'instance',
-            metavar='<instance>',
-            help='Instance name or ID',
-        )
-        return parser
-
-    def take_action(self, parsed_args):
-        uri = '/instances/%s/state-transitions' % parsed_args.instance
-        self.app.session.post(uri, payload=dict(new_state="SUSPENDED"))
-
-
-class ResumeInstance(Command):
-    """Resume instance."""
-
-    def get_parser(self, prog_name):
-        parser = super(ResumeInstance, self).get_parser(prog_name)
-        parser.add_argument(
-            'instance',
-            metavar='<instance>',
-            help='Instance name or ID',
-        )
-        return parser
-
-    def take_action(self, parsed_args):
-        uri = '/instances/%s/state-transitions' % parsed_args.instance
-        self.app.session.post(uri, payload=dict(new_state="ACTIVE"))
-
-
-class RebootInstance(Command):
-    """Reboot instance."""
-
-    def get_parser(self, prog_name):
-        parser = super(RebootInstance, self).get_parser(prog_name)
-        parser.add_argument(
-            'instance',
-            metavar='<instance>',
-            help='Instance name or ID',
-        )
-
-        parser.add_argument(
-            '--type',
-            default='SOFT',
-            metavar='<type>',
-            help='either `SOFT` for a software-level reboot, or `HARD` for a virtual '
-                 'power cycle hard reboot',
-            choices=['SOFT', 'HARD']
-        )
-        return parser
-
-    def take_action(self, parsed_args):
-        uri = '/instances/%s/state-transitions' % parsed_args.instance
-        self.app.session.post(uri, payload=dict(new_state="REBOOT",
-                                                params={"reboot_type": parsed_args.type}))
+    def confirm_volume_deletion(self, parsed_args):
+        if parsed_args.delete_root_volume:
+            return True
+        if is_tty():
+            return Confirm.ask(
+                "Would you like to also delete the root volume of this instance?",
+                default=False
+            )
+        return False
 
 
 class ListInstanceVolume(Lister):
     """List instance volume(s)."""
+    schema = schemas.InstanceVolume(many=True)
 
     def get_parser(self, prog_name):
         parser = super(ListInstanceVolume, self).get_parser(prog_name)
@@ -373,15 +292,29 @@ class ListInstanceVolume(Lister):
 
         return parser
 
-    def take_action(self, parsed_args):
-        uri = '/instances/%s/volumes' % parsed_args.instance
-        data = self.app.session.get(uri)
+    def get_data(self, parsed_args):
+        # TODO: Could use a little bit of cache maybe
+        # TODO: Duplicate logic
+        volumes = {
+            v['id']: v for v in self.app.session.get('/volumes')
+        }
+        instance_info = self.app.session.get('/instances/%s' % parsed_args.instance)
+        uri = '/instances/%s/volumes' % instance_info['id']
+        instance_volumes = self.app.session.get(uri)
+        return [
+            {
+                "instance": instance_info['name'],
+                "volume": volumes[volume['id']]['name'],
+                **volume
+            }
+            for volume in instance_volumes
 
-        return tabulate(data)
+        ]
 
 
 class ShowInstanceVolume(ShowOne):
     """Show instance volume details."""
+    schema = schemas.InstanceVolume(many=False)
 
     def get_parser(self, prog_name):
         parser = super(ShowInstanceVolume, self).get_parser(prog_name)
@@ -398,15 +331,25 @@ class ShowInstanceVolume(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
+        # TODO: Could use a little bit of cache maybe
+        # TODO: Duplicate logic
+        volumes = {
+            v['id']: v for v in self.app.session.get('/volumes')
+        }
+        instance_info = self.app.session.get('/instances/%s' % parsed_args.instance)
         uri = '/instances/%s/volumes/%s' % (parsed_args.instance, parsed_args.volume)
-        data = self.app.session.get(uri)
-
-        return tabulate(data)
+        instance_volume = self.app.session.get(uri)
+        return {
+            "instance": instance_info['name'],
+            "volume": volumes[instance_volume['id']]['name'],
+            **instance_volume
+        }
 
 
 class AttachInstanceVolume(ShowOne):
     """Attach instance volume."""
+    schema = schemas.InstanceVolume(many=False)
 
     def get_parser(self, prog_name):
         parser = super(AttachInstanceVolume, self).get_parser(prog_name)
@@ -424,13 +367,13 @@ class AttachInstanceVolume(ShowOne):
 
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/instances/%s/volumes' % parsed_args.instance
         payload = {'id': parsed_args.volume}
 
         data = self.app.session.post(uri, payload)
 
-        return tabulate(data)
+        return data
 
 
 class DetachInstanceVolume(Command):
@@ -453,12 +396,13 @@ class DetachInstanceVolume(Command):
         return parser
 
     def take_action(self, parsed_args):
-        uri = '/instances/%s/volumes/%s' %(parsed_args.instance, parsed_args.volume)
+        uri = '/instances/%s/volumes/%s' % (parsed_args.instance, parsed_args.volume)
         self.app.session.delete(uri)
 
 
 class ListInstanceInterface(Lister):
     """List instance interface(s)."""
+    schema = schemas.InstanceInterface(many=True)
 
     def get_parser(self, prog_name):
         parser = super(ListInstanceInterface, self).get_parser(prog_name)
@@ -470,15 +414,16 @@ class ListInstanceInterface(Lister):
 
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/instances/%s/interfaces' % parsed_args.instance
         data = self.app.session.get(uri)
 
-        return tabulate(data)
+        return data
 
 
 class AttachInterface(ShowOne):
     """Attach interface."""
+    schema = schemas.InstanceInterface(many=False)
 
     def get_parser(self, prog_name):
         parser = super(AttachInterface, self).get_parser(prog_name)
@@ -496,11 +441,11 @@ class AttachInterface(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         payload = {'net': parsed_args.network}
         uri = '/instances/%s/interfaces' % parsed_args.instance
         data = self.app.session.post(uri, payload)
-        return tabulate(data)
+        return data
 
 
 class DetachInterface(Command):
@@ -524,12 +469,15 @@ class DetachInterface(Command):
         return parser
 
     def take_action(self, parsed_args):
-        uri = '/instances/%s/interfaces/%s' %(parsed_args.instance, parsed_args.interface_id)
+        uri = '/instances/%s/interfaces/%s' % (
+            parsed_args.instance, parsed_args.interface_id)
         self.app.session.delete(uri)
 
 
 class UpdateInstance(ShowOne):
-    """Update instance name."""
+    """Update instance name, description or even resize it to another instance-type"""
+    schema = schemas.InstanceDetails()
+
     def get_parser(self, prog_name):
         parser = super(UpdateInstance, self).get_parser(prog_name)
         parser.add_argument(
@@ -541,36 +489,113 @@ class UpdateInstance(ShowOne):
             '--name',
             metavar='<name>',
             required=False,
-            help=('Instance new name, should be unique'),
+            help='Instance new name, must be unique',
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
             required=False,
-            help=('Instance description'),
+            help='Instance description',
+        )
+        parser.add_argument(
+            '--instance-type',
+            metavar='<instance_type>',
+            required=False,
+            help='New instance type you want to use for this instance',
         )
 
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/instances/%s' % parsed_args.instance
         payload = {}
         if parsed_args.name:
             payload['name'] = parsed_args.name
         if parsed_args.description:
             payload['description'] = parsed_args.description
+        if parsed_args.instance_type:
+            payload['instance_type'] = parsed_args.instance_type
+            self.app.console.print(
+                "Resizing your instance might cause additional charges",
+                style='bold yellow'
+            )
         if not payload:
-            raise Exception("At least one of --name or --description should be provided")
+            raise CLIError([
+                dict(
+                    msg="At least one of --name or --description or "
+                        "--instance-type must be provided"
+                )
+            ]
+            )
         data = self.app.session.patch(uri, payload)
 
-        return tabulate(data)
+        return data
+
+    def _on_400(self, parsed_args, response):
+        error_msg = response.json()['errors']
+        if 'stop your instance' in error_msg:
+            return CLIError([
+                dict(
+                    msg=error_msg,
+                    hint="You can stop your instance by running: "
+                         "[bold blue]digicloud instance stop {}[/bold blue]".format(
+                        parsed_args.instance
+                    )
+                )
+            ])
 
 
-class ListSecurityGroup(Lister):
-    """List Instance Security Groups"""
+class ResizeInstance(ShowOne):
+    """Resize your instance"""
+    schema = schemas.InstanceDetails()
 
     def get_parser(self, prog_name):
-        parser = super(ListSecurityGroup, self).get_parser(prog_name)
+        parser = super(ResizeInstance, self).get_parser(prog_name)
+        parser.add_argument(
+            'instance',
+            metavar='<instance>',
+            help='Instance ID or name',
+        )
+        parser.add_argument(
+            '--instance-type',
+            metavar='<instance_type>',
+            required=True,
+            help='The instance type you want to use for this instance',
+        )
+
+        return parser
+
+    def get_data(self, parsed_args):
+        uri = '/instances/%s' % parsed_args.instance
+        payload = {}
+        if parsed_args.instance_type:
+            payload['instance_type'] = parsed_args.instance_type
+            self.app.console.print(
+                "Resizing your instance might cause additional charges",
+                style='bold yellow'
+            )
+        return self.app.session.patch(uri, payload)
+
+    def _on_400(self, parsed_args, response):
+        error_msg = response.json()['errors']
+        if 'stop your instance' in error_msg:
+            return CLIError([
+                dict(
+                    msg=error_msg,
+                    hint="You can stop your instance by running: "
+                         "[bold blue]digicloud instance stop {}[/bold blue]".format(
+                        parsed_args.instance
+                    )
+                )
+            ])
+
+
+class ListFirewall(Lister):
+    """List Instance Firewalls"""
+    schema = schemas.FirewallList(many=True)
+
+    def get_parser(self, prog_name):
+        parser = super(ListFirewall, self).get_parser(prog_name)
         parser.add_argument(
             'instance',
             metavar='<instance>',
@@ -578,55 +603,55 @@ class ListSecurityGroup(Lister):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/instances/%s/security-groups' % parsed_args.instance
         data = self.app.session.get(uri)
-        return tabulate(data)
+        return data
 
 
-class AddSecurityGroup(Command):
-    """Add Security Group to instance"""
+class AddFirewall(Command):
+    """Add Firewall to instance"""
 
     def get_parser(self, prog_name):
-        parser = super(AddSecurityGroup, self).get_parser(prog_name)
+        parser = super(AddFirewall, self).get_parser(prog_name)
         parser.add_argument(
             'instance',
             metavar='<instance>',
             help='Instance name or ID'
         )
         parser.add_argument(
-            '--security-group',
+            '--firewall',
             required=True,
-            metavar='<security_group>',
-            help='Security group name or ID',
+            metavar='<firewall>',
+            help='Firewall name or ID',
         )
         return parser
 
     def take_action(self, parsed_args):
-        payload = {'security_group_ref': parsed_args.security_group}
+        payload = {'security_group_ref': parsed_args.firewall}
         uri = '/instances/%s/security-groups' % parsed_args.instance
         self.app.session.post(uri, payload)
 
 
-class RemoveSecurityGroup(Command):
-    """Remove Security Group from Instance"""
+class RemoveFirewall(Command):
+    """Remove Firewall from Instance"""
 
     def get_parser(self, prog_name):
-        parser = super(RemoveSecurityGroup, self).get_parser(prog_name)
+        parser = super(RemoveFirewall, self).get_parser(prog_name)
         parser.add_argument(
             'instance',
             metavar='<instance>',
             help='Instance name or ID'
         )
         parser.add_argument(
-            '--security-group',
+            '--firewall',
             required=True,
-            metavar='<security_group>',
-            help='Security group name or ID',
+            metavar='<firewall>',
+            help='Firewall name or ID',
         )
         return parser
 
     def take_action(self, parsed_args):
         uri = '/instances/%s/security-groups/%s' % (parsed_args.instance,
-                                                    parsed_args.security_group)
+                                                    parsed_args.firewall)
         self.app.session.delete(uri)
