@@ -1,24 +1,25 @@
 """
     DigiCloud Router Service.
 """
-from cliff.command import Command
-from cliff.lister import Lister
-from cliff.show import ShowOne
 
-from ..utils import tabulate
+from digicloud import schemas
+from .base import Lister, ShowOne, Command
+from ..error_handlers import CLIError
 
 
 class ListRouter(Lister):
     """List Routers."""
+    schema = schemas.RouterList(many=True)
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         data = self.app.session.get('/routers')
 
-        return tabulate(data)
+        return data
 
 
 class ShowRouter(ShowOne):
     """Show Router details."""
+    schema = schemas.RouterDetails(many=False)
 
     def get_parser(self, prog_name):
         parser = super(ShowRouter, self).get_parser(prog_name)
@@ -29,11 +30,11 @@ class ShowRouter(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/routers/%s' % parsed_args.router
         data = self.app.session.get(uri)
 
-        return tabulate(data)
+        return data
 
 
 class DeleteRouter(Command):
@@ -55,6 +56,7 @@ class DeleteRouter(Command):
 
 class CreateRouter(ShowOne):
     """Create Router."""
+    schema = schemas.RouterDetails(many=False)
 
     def get_parser(self, prog_name):
         parser = super(CreateRouter, self).get_parser(prog_name)
@@ -66,22 +68,25 @@ class CreateRouter(ShowOne):
         parser.add_argument(
             '--admin-state',
             metavar='<AdminState>',
-            default='UP',
-            help='Set admin state (default UP).'
+            choices=['UP', 'DOWN'],
+            type=str.upper,
+            help='Set admin state, could be "UP" or "DOWN", default "UP"'
         )
+
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         payload = {
             'name': parsed_args.name,
-            'admin_state': parsed_args.admin_state
+            'admin_state': parsed_args.admin_state,
         }
         data = self.app.session.post('/routers', payload)
-        return tabulate(data)
+        return data
 
 
 class ListRouterInterface(Lister):
     """List all router interfaces"""
+    schema = schemas.RouterInterfaceList(many=True)
 
     def get_parser(self, prog_name):
         parser = super(ListRouterInterface, self).get_parser(prog_name)
@@ -92,14 +97,16 @@ class ListRouterInterface(Lister):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = f'/routers/{parsed_args.router}/interfaces'
         data = self.app.session.get(uri)
-        return tabulate(data)
+        return data
 
 
 class ShowRouterInterface(ShowOne):
     """Show router interface."""
+    schema = schemas.RouterInterfaceDetails(many=False)
+
     def get_parser(self, prog_name):
         parser = super(ShowRouterInterface, self).get_parser(prog_name)
         parser.add_argument(
@@ -115,15 +122,16 @@ class ShowRouterInterface(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/routers/%s/interfaces/%s' % (parsed_args.router, parsed_args.interface_id)
         data = self.app.session.get(uri)
 
-        return tabulate(data)
+        return data
 
 
 class AddRouterInterface(ShowOne):
     """Add router interface."""
+    schema = schemas.RouterInterfaceDetails(many=False)
 
     def get_parser(self, prog_name):
         parser = super(AddRouterInterface, self).get_parser(prog_name)
@@ -136,16 +144,16 @@ class AddRouterInterface(ShowOne):
             '--subnet',
             required=True,
             metavar='<subnet>',
-            help='Subnet name or ID'
+            help='SubnetList name or ID'
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/routers/%s/interfaces' % parsed_args.router
         payload = {'subnet': parsed_args.subnet}
         data = self.app.session.post(uri, payload)
 
-        return tabulate(data)
+        return data
 
 
 class RemoveRouterInterface(Command):
@@ -172,6 +180,7 @@ class RemoveRouterInterface(Command):
 
 class AddRouterExternal(ShowOne):
     """Add external network to router."""
+    schema = schemas.RouterDetails(many=False)
 
     def get_parser(self, prog_name):
         parser = super(AddRouterExternal, self).get_parser(prog_name)
@@ -182,15 +191,16 @@ class AddRouterExternal(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/routers/%s' % parsed_args.router
         payload = {'has_gateway': True}
         data = self.app.session.patch(uri, payload)
-        return tabulate(data)
+        return data
 
 
 class RemoveRouterExternal(ShowOne):
     """Remove external network from router."""
+    schema = schemas.RouterDetails(many=False)
 
     def get_parser(self, prog_name):
         parser = super(RemoveRouterExternal, self).get_parser(prog_name)
@@ -201,15 +211,16 @@ class RemoveRouterExternal(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/routers/%s' % parsed_args.router
         payload = {'has_gateway': False}
         data = self.app.session.patch(uri, payload)
-        return tabulate(data)
+        return data
 
 
 class UpdateRouter(ShowOne):
     _description = "Update Router"
+    schema = schemas.RouterDetails(many=False)
 
     def get_parser(self, prog_name):
         parser = super(UpdateRouter, self).get_parser(prog_name)
@@ -227,8 +238,11 @@ class UpdateRouter(ShowOne):
         parser.add_argument(
             '--admin-state',
             metavar='<AdminState>',
-            help='Set admin state.'
+            choices=['UP', 'DOWN'],
+            type=str.upper,
+            help='Set admin state, could be "UP" or "DOWN"'
         )
+
         group.add_argument(
             '--disable-gateway',
             help='Disconnect router from external network',
@@ -242,7 +256,7 @@ class UpdateRouter(ShowOne):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         payload = {}
         if parsed_args.name:
             payload['name'] = parsed_args.name
@@ -253,14 +267,18 @@ class UpdateRouter(ShowOne):
         if parsed_args.admin_state:
             payload['admin_state'] = parsed_args.admin_state
         if len(payload) == 0:
-            raise Exception("You need to at least provide one of --name, --enable-gateway or --disable-gateway")
+            raise CLIError([dict(
+                msg="You need to at least provide one of "
+                    "--name, --admin-state, --enable-gateway or --disable-gateway"
+            )])
         uri = '/routers/%s' % parsed_args.router
         data = self.app.session.patch(uri, payload)
-        return tabulate(data)
+        return data
 
 
 class ListRouterStatic(Lister):
     """List router static routes."""
+
     def get_parser(self, prog_name):
         parser = super(ListRouterStatic, self).get_parser(prog_name)
         parser.add_argument(
@@ -270,10 +288,10 @@ class ListRouterStatic(Lister):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/routers/%s/static-routes' % parsed_args.router_id
         data = self.app.session.get(uri)
-        return tabulate(data)
+        return data
 
 
 class AddRouterStatic(Lister):
@@ -299,7 +317,7 @@ class AddRouterStatic(Lister):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/routers/%s/static-routes' % parsed_args.router_id
         payload = {
             "destination": parsed_args.destination,
@@ -307,7 +325,7 @@ class AddRouterStatic(Lister):
 
         }
         data = self.app.session.post(uri, payload)
-        return tabulate(data)
+        return data
 
 
 class DeleteRouterStatic(Lister):
@@ -333,7 +351,7 @@ class DeleteRouterStatic(Lister):
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/routers/%s/static-routes' % parsed_args.router_id
         payload = {
             "destination": parsed_args.destination,
@@ -341,4 +359,4 @@ class DeleteRouterStatic(Lister):
 
         }
         data = self.app.session.delete(uri, payload)
-        return tabulate(data)
+        return data
