@@ -1,111 +1,115 @@
 """
     DigiCloud Security Service.
 """
-from cliff.command import Command
-from cliff.lister import Lister
-from cliff.show import ShowOne
 
-from ..utils import tabulate
+from .base import Lister, ShowOne, Command
+from .. import schemas
 
 
-class ListSecurityGroup(Lister):
-    """List Security Groups."""
+class ListFirewall(Lister):
+    """List Firewalls."""
+    schema = schemas.FirewallList(many=True)
+    help_file = 'firewall.txt'
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         data = self.app.session.get('/security-groups')
 
-        return tabulate(data)
+        return data
 
 
-class ShowSecurityGroup(ShowOne):
-    """Show security group details."""
+class ShowFirewall(ShowOne):
+    """Show Firewall details."""
+    schema = schemas.FirewallDetails()
+    help_file = 'firewall.txt'
 
     def get_parser(self, prog_name):
-        parser = super(ShowSecurityGroup, self).get_parser(prog_name)
+        parser = super(ShowFirewall, self).get_parser(prog_name)
         parser.add_argument(
-            'security_group',
-            metavar='<security_group>',
-            help='Security group name or ID',
+            'firewall',
+            metavar='<firewall>',
+            help='firewall name or ID',
         )
         return parser
 
-    def take_action(self, parsed_args):
-        uri = '/security-groups/%s' % parsed_args.security_group
+    def get_data(self, parsed_args):
+        uri = '/security-groups/%s' % parsed_args.firewall
         data = self.app.session.get(uri)
 
-        return tabulate(data)
+        return data
 
 
-class CreateSecurityGroup(ShowOne):
-    """Create Security Group."""
+class CreateFirewall(ShowOne):
+    """Create Firewall."""
+    schema = schemas.FirewallDetails()
+    help_file = 'firewall.txt'
 
     def get_parser(self, prog_name):
-        parser = super(CreateSecurityGroup, self).get_parser(prog_name)
+        parser = super(CreateFirewall, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<name>',
-            help='Security group name'
+            help='Firewall name'
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
             default="",
             required=False,
-            help='Security group name'
+            help='Firewall description'
         )
 
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         payload = {'name': parsed_args.name, 'description': parsed_args.description}
         data = self.app.session.post('/security-groups', payload)
 
-        return tabulate(data)
+        return data
 
 
-class DeleteSecurityGroup(Command):
-    """Delete Security Group."""
+class DeleteFirewall(Command):
+    """Delete Firewall."""
 
     def get_parser(self, prog_name):
-        parser = super(DeleteSecurityGroup, self).get_parser(prog_name)
+        parser = super(DeleteFirewall, self).get_parser(prog_name)
         parser.add_argument(
-            'security_group',
-            metavar='<security_group>',
-            help='Security group name or ID'
+            'firewall',
+            metavar='<firewall>',
+            help='Firewall name or ID'
         )
         return parser
 
     def take_action(self, parsed_args):
-        uri = '/security-groups/%s' % parsed_args.security_group
+        uri = '/security-groups/%s' % parsed_args.firewall
         self.app.session.delete(uri)
 
 
-class UpdateSecurityGroup(Command):
-    """Update Security Group."""
+class UpdateFirewall(Command):
+    """Update Firewall."""
 
     def get_parser(self, prog_name):
-        parser = super(UpdateSecurityGroup, self).get_parser(prog_name)
+        parser = super(UpdateFirewall, self).get_parser(prog_name)
         parser.add_argument(
-            'security_group',
-            metavar='<security_group>',
-            help='Security group name or ID'
+            'firewall',
+            metavar='<firewall>',
+            help='Firewall name or ID'
         )
         parser.add_argument(
             '--name',
             metavar='<name>',
             required=False,
-            help='Security group name'
+            help='Firewall name'
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
             required=False,
-            help='Security group name'
+            help='Firewall description'
         )
         return parser
 
     def take_action(self, parsed_args):
-        uri = f'/security-groups/{parsed_args.security_group}'
+        uri = f'/security-groups/{parsed_args.firewall}'
         payload = {}
         if parsed_args.name:
             payload['name'] = parsed_args.name
@@ -114,61 +118,67 @@ class UpdateSecurityGroup(Command):
         self.app.session.patch(uri, payload)
 
 
-class ListSecurityGroupRule(Lister):
-    """List Security Group Rules."""
+class ListFirewallRule(Lister):
+    """List Firewall Rules."""
+    schema = schemas.FirewallRuleList(many=True)
+    help_file = 'firewall.txt'
 
     def get_parser(self, prog_name):
-        parser = super(ListSecurityGroupRule, self).get_parser(prog_name)
+        parser = super(ListFirewallRule, self).get_parser(prog_name)
         parser.add_argument(
-            'security_group',
-            metavar='<security_group>',
-            help='Security group rule ID',
+            'firewall',
+            metavar='<firewall>',
+            help='Firewall rule ID',
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         data = self.app.session.get(
-            '/security-groups/{}/rules'.format(parsed_args.security_group)
+            '/security-groups/{}/rules'.format(parsed_args.firewall)
         )
-        return tabulate(data)
+        return data
 
 
-class ShowSecurityGroupRule(ShowOne):
-    """Show security group rule details."""
+class ShowFirewallRule(ShowOne):
+    """Show Firewall rule details."""
+    schema = schemas.FirewallRuleDetails()
+    help_file = 'firewall.txt'
 
     def get_parser(self, prog_name):
-        parser = super(ShowSecurityGroupRule, self).get_parser(prog_name)
+        parser = super(ShowFirewallRule, self).get_parser(prog_name)
         parser.add_argument(
-            'security_group_rule',
-            metavar='<security_group_rule>',
-            help='Security group rule ID',
+            'firewall_rule',
+            metavar='<firewall_rule>',
+            help='Firewall rule ID',
         )
         parser.add_argument(
-            '--security-group',
+            '--firewall',
             required=True,
-            metavar='<security_group>',
-            help='security group name or ID'
+            metavar='<firewall>',
+            help='Firewall name or ID'
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         uri = '/security-groups/{}/rules/{}'.format(
-            parsed_args.security_group,
-            parsed_args.security_group_rule
+            parsed_args.firewall,
+            parsed_args.firewall_rule
         )
         data = self.app.session.get(uri)
-        return tabulate(data)
+        return data
 
 
-class CreateSecurityGroupRule(ShowOne):
-    """Create Security Group Rule."""
+class CreateFirewallRule(ShowOne):
+    """Create Firewall Rule."""
+    schema = schemas.FirewallRuleDetails()
+    help_file = 'firewall.txt'
 
     def get_parser(self, prog_name):
-        parser = super(CreateSecurityGroupRule, self).get_parser(prog_name)
+        parser = super(CreateFirewallRule, self).get_parser(prog_name)
         parser.add_argument(
-            'security_group',
-            metavar='<security_group>',
-            help='Security group name or ID'
+            'firewall',
+            metavar='<firewall>',
+            help='Firewall name or ID'
         )
         parser.add_argument(
             '--direction',
@@ -203,42 +213,42 @@ class CreateSecurityGroupRule(ShowOne):
             '--remote-ip-prefix',
             required=False,
             metavar='<remote_ip_prefix>',
-            help='remote IP prefix matched by this security group rule'
+            help='remote IP prefix matched by this Firewall rule'
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
         args = [
             'direction', 'ethertype', 'port_range_min',
             'port_range_max', 'protocol', 'remote_ip_prefix',
         ]
         payload = {arg: getattr(parsed_args, arg) for arg in args
                    if getattr(parsed_args, arg) is not None}
-        uri = '/security-groups/%s/rules' % parsed_args.security_group
+        uri = '/security-groups/%s/rules' % parsed_args.firewall
         data = self.app.session.post(uri, payload)
 
-        return tabulate(data)
+        return data
 
 
-class DeleteSecurityGroupRule(Command):
-    """Delete Security Group Rule."""
+class DeleteFirewallRule(Command):
+    """Delete Firewall Rule."""
 
     def get_parser(self, prog_name):
-        parser = super(DeleteSecurityGroupRule, self).get_parser(prog_name)
+        parser = super(DeleteFirewallRule, self).get_parser(prog_name)
         parser.add_argument(
-            'security_group_rule',
-            metavar='<security_group_rule>',
-            help='Security group rule ID'
+            'firewall_rule',
+            metavar='<firewall_rule>',
+            help='Firewall rule ID'
         )
         parser.add_argument(
-            '--security-group',
+            '--firewall',
             required=True,
-            metavar='<security_group>',
-            help='Security group name or ID'
+            metavar='<firewall>',
+            help='Firewall name or ID'
         )
         return parser
 
     def take_action(self, parsed_args):
-        uri = '/security-groups/{}/rules/{}'.format(parsed_args.security_group,
-                                                    parsed_args.security_group_rule)
+        uri = '/security-groups/{}/rules/{}'.format(parsed_args.firewall,
+                                                    parsed_args.firewall_rule)
         self.app.session.delete(uri)
